import java.util.*;

/** 
  Realizza il tipo di dato Coda (di interi) con le usuali operazioni. 
  Implementazione mediante array circolare, seguendo lo schema utilizzato 
  per la classe Stack vista a lezione.  

  Invariante: 

    . gli elementi della coda occupano le posizioni da [front+1 .. rear]. 
    . la posizione front e sempre libera (sacrifichiamo una posizione per 
      semplificare i test di coda piena e coda vuota
*/

class Queue 
{
    private int[] contents; 
    private int front, rear;
    private int size;

    public Queue(int n)
    {
	size = n;
	front =  rear = 0;
	contents = new int[size];
    }

    public boolean empty()
    {
	return front == rear; 
    }

    private boolean full()
    {
	return (rear+1) % size == front; 
    }

    public int dequeue() 
    {
	if (empty()) throw new NoSuchElementException();
	front = (front +1) % size; 
	return contents[front];
    }
	    
    public void enqueue(int el)
    {
	if (full()) 
	    {
		resize(); enqueue(el); return;
	    }

	rear = (rear + 1) % size; 
	contents[rear] = el; 
    }

    private void resize()
    {
	int[] tmp = new int[size * 2];
	int j = 1; 
	for (int i = (front+1) % size; i != rear; i = (i+1) % size, j++)
	    tmp[j] = contents[i];
	tmp[j] = contents[rear];
	contents = tmp;
	front = 0; rear = j; size = size * 2; 
    }

    public String toString()
    {
	if (empty()) return "";

	String out = "";
	for (int i = (front+1) % size; i != rear; i = (i+1) % size)
	    out += contents[i]+" ";
	return out + contents[rear];
    }
}